//+------------------------------------------------------------------+
//|                                                   SignalMACD.mqh |
//|                   Copyright 2009-2013, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include "KKInfo.mqh"
#include "ExpertSignal.mqh"   //#include <Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Signals of oscillator 'mutiframeMACD'                      |
//| Type=SignalAdvanced                                              |
//| Name=MTF_SAR                                                    |
//| ShortName=MTF_SAR                                               |
//| Class=CSignalSAR                                              |
//| Page=signal_muti_SAR                                            |
//| Parameter=Applied,ENUM_APPLIED_PRICE,PRICE_CLOSE,Prices series   |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| Class CSignalSAR.                                             |
//| Purpose: Class of generator of trade signals based on            |
//|          the 'Moving Average Convergence/Divergence' oscillator. |
//| Is derived from the CExpertSignal class.                         |
//+------------------------------------------------------------------+
class CSignalSAR : public CExpertSignal
  {
protected:
   //CiSAR            m_SAR;           // object-oscillator
   CiCustom          m_CiCustom_SAR;          // // object-oscillator
  // CiCustom          m_CiCustom2_SAR;          // object-oscillator
   int ci_handle1;


   int               filter_ma;
   ENUM_MT_POLICY    m1_state;           // object-oscillator
   //  ENUM_MT_POLICY    m2_state;          // object-oscillator
   //--- adjusted parameters
   ENUM_TIMEFRAMES   m_period_op ;   // SAR operation Period
  // int             m_period_small;
   //int             m_period_big;
   ENUM_APPLIED_PRICE m_applied;       // the "price series" parameter of the oscillator
   //--- "weights" of market models (0-100)
   int               m_pattern_0;      // model 0 "the oscillator has required direction"
   int               m_pattern_1;      // model 1 "reverse of the oscillator to required direction"
   int               m_pattern_2;      // model 2 "crossing of main and signal line"
   int               m_pattern_3;      // model 3 "crossing of main line an the zero level"
   int               m_pattern_4;      // model 4 "divergence of the oscillator and price"
   int               m_pattern_5;      // model 5 "double divergence of the oscillator and price"
   //--- variables
   double            m_extr_osc[10];   // array of values of extremums of the oscillator
   double            m_extr_pr[10];    // array of values of the corresponding extremums of price
   int               m_extr_pos[10];   // array of shifts of extremums (in bars)
   uint              m_extr_map;       // resulting bit-map of ratio of extremums of the oscillator and the price
   double            pass_take_level;
   double             _state[];
   double            _ma1[];
   double            _ma2[];
   
public:
                     CSignalSAR(void);
                    ~CSignalSAR(void);
   //--- methods of setting adjustable parameters
   void              PeriodOperation(ENUM_TIMEFRAMES value){ m_period_op    =value;     }
   void              Applied(ENUM_APPLIED_PRICE value) { m_applied=value;               }
   void              FilterEMA(int value)                { filter_ma =value; }
   
      //--- methods for generating signals of entering the market
   virtual bool      CheckOpenLong(double &price,double &sl,double &tp,datetime &expiration);
   virtual bool      CheckOpenShort(double &price,double &sl,double &tp,datetime &expiration);
      //--- methods for generating signals of reversal of positions
   virtual ENUM_MT_POLICY   MTF_CheckReverseLong (double &price,double &sl,double &tp,datetime &expiration);
   virtual ENUM_MT_POLICY   MTF_CheckReverseShort(double &price,double &sl,double &tp,datetime &expiration);
      //--- methods for generating signals of exit from the market
   virtual bool      CheckCloseLong(double &price);
   virtual bool      CheckCloseShort(double &price);
   virtual bool      CloseLong(double &price);
   virtual bool      CloseShort(double &price);
   //--- methods of adjusting "weights" of market models
   void              Pattern_0(int value)              { m_pattern_0=value;             }
   void              Pattern_1(int value)              { m_pattern_1=value;             }
   void              Pattern_2(int value)              { m_pattern_2=value;             }
   void              Pattern_3(int value)              { m_pattern_3=value;             }
   void              Pattern_4(int value)              { m_pattern_4=value;             }
   void              Pattern_5(int value)              { m_pattern_5=value;             }
   //--- method of verification of settings
   virtual bool      ValidationSettings(void);
   //--- method of creating the indicator and timeseries
   virtual bool      InitIndicators(CIndicators *indicators);
   //--- methods of checking if the market models are formed
   //virtual int       ShortCondition(void);
   
  // bool update();//run process
protected:
   //--- method of initialization of the oscillator
   bool              InitSAR(CIndicators *indicators,ENUM_TIMEFRAMES my_period);
   //--- methods of getting data
/*
   double            BuyBuffer(int ind)      { return(m_CiCustom_SAR.GetData(2,ind));    }
   double            SellBuffer(int ind)     { return(m_CiCustom_SAR.GetData(3,ind));    }
   double            UpSarBuffer(int ind)    { return(m_CiCustom_SAR.GetData(0,ind));    }
   double            DnSarBuffer(int ind)    { return(m_CiCustom_SAR.GetData(1,ind));    }
  */ 
  // double            State(int ind) { return(Main(ind)-Signal(ind); }
   bool              ExtState(int ind);
   bool              CompareMaps(int map,int count,bool minimax=false,int start=0);
  };
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CSignalSAR::CSignalSAR(void) : m_period_op(PERIOD_CURRENT),
                                // share_ma_handle(INVALID_HANDLE),
                                 filter_ma(12),
                       
                                 m_applied(PRICE_CLOSE),
                                 m_pattern_0(10),
                                 m_pattern_1(30),
                                 m_pattern_2(80),
                                 m_pattern_3(50),
                                 m_pattern_4(60),
                                 m_pattern_5(100)
  {
   //--- initialization of protected data
   m_used_series=USE_SERIES_HIGH+USE_SERIES_LOW;
   pass_take_level=m_take_level;
  }
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CSignalSAR::~CSignalSAR(void)
  {
  }
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//+------------------------------------------------------------------+
bool CSignalSAR::ValidationSettings(void)
  {
//--- validation settings of additional filters
   if(!CExpertSignal::ValidationSettings())
      return(false);
//--- initial data checks


//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//+------------------------------------------------------------------+
bool CSignalSAR::InitIndicators(CIndicators *indicators)
  {
//--- check of pointer is performed in the method of the parent class
//---
//--- initialization of indicators and timeseries of additional filters
   if(!CExpertSignal::InitIndicators(indicators))
      return(false);
//--- create and initialize SAR oscilator
   if(!InitSAR(indicators,m_period_op))
      return(false);
   if(share_ma_handle==INVALID_HANDLE && m_period_op < PERIOD_H1)
      if((share_ma_handle=iMA(NULL,0,filter_ma,0,MODE_EMA,PRICE_CLOSE))==INVALID_HANDLE)
        {
         printf("Error creating EMA indicator");
         return(false);
        }   
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Initialize SAR oscillators.                                     |
//+------------------------------------------------------------------+
bool CSignalSAR::InitSAR(CIndicators *indicators,ENUM_TIMEFRAMES my_period)
  {
/*input double Junior_Step=0.02;    // Junior iSAR step
input double Junior_Maximum=0.2;  // Junior iSAR maximum
//---- middle timeframe iSAR indicator parameters
input ENUM_TIMEFRAMES Middle_TimeFrame=PERIOD_H1; // Middle iSAR chart period
input double Middle_Step=0.02;    // Middle iSAR step
input double Middle_Maximum=0.2;  // Middle iSAR maximum
//---- senior timeframe iSAR indicator parameters
input ENUM_TIMEFRAMES Senior_TimeFrame=PERIOD_H12; // Senior iSAR chart period
input double Senior_Step=0.02;    // Senior iSAR step
input double Senior_Maximum=0.2;  // Senior iSAR maximum
*/
   //--- initialize object
//=============================================

  // if(!m_CiCustom_SAR.NumBuffers(4)) 
  //    return(false);
   return(true);
  }

//+------------------------------------------------------------------+
//| Extended check of the oscillator state consists                  |
//| in forming a bit-map according to certain rules,                 |
//| which shows ratios of extremums of the oscillator and price.     |
//+------------------------------------------------------------------+
bool CSignalSAR::ExtState(int ind)
  {
//--- operation of this method results in a bit-map of extremums
//--- practically, the bit-map of extremums is an "array" of 4-bit fields
//--- each "element of the array" definitely describes the ratio
//--- of current extremums of the oscillator and the price with previous ones
//--- purpose of bits of an element of the analyzed bit-map
//--- bit 3 - not used (always 0)
//--- bit 2 - is equal to 1 if the current extremum of the oscillator is "more extreme" than the previous one
//---         (a higher peak or a deeper valley), otherwise - 0
//--- bit 1 - not used (always 0)
//--- bit 0 - is equal to 1 if the current extremum of price is "more extreme" than the previous one
//---         (a higher peak or a deeper valley), otherwise - 0
//--- in addition to them, the following is formed:
//--- array of values of extremums of the oscillator,
//--- array of values of price extremums and
//--- array of "distances" between extremums of the oscillator (in bars)
//--- it should be noted that when using the results of the extended check of state,
//--- you should consider, which extremum of the oscillator (peak or valley)
//--- is the "reference point" (i.e. was detected first during the analysis)
//--- if a peak is detected first then even elements of all arrays
//--- will contain information about peaks, and odd elements will contain information about valleys
//--- if a valley is detected first, then respectively in reverse
   int    pos=ind,off=0,index;
   uint   map;                 // intermediate bit-map for one extremum
//---
   m_extr_map=0;
   for(int i=0;i<10;i++)
     {
    //  off=StateMain(pos);
      if(off>0)
        {
         //--- minimum of the oscillator is detected
         pos+=off;
         m_extr_pos[i]=pos;
         if(i>1)
           {
            m_extr_pr[i]=m_low.MinValue(pos-2,5,index);
            //--- form the intermediate bit-map
            map=0;
            if(m_extr_pr[i-2]<m_extr_pr[i])
               map+=1;  // set bit 0
            if(m_extr_osc[i-2]<m_extr_osc[i])
               map+=4;  // set bit 2
            //--- add the result
            m_extr_map+=map<<(4*(i-2));
           }
         else
            m_extr_pr[i]=m_low.MinValue(pos-1,4,index);
        }
      else
        {
         //--- maximum of the oscillator is detected
         pos-=off;
         m_extr_pos[i]=pos;
         //m_extr_osc[i]=Main(pos);
         if(i>1)
           {
            m_extr_pr[i]=m_high.MaxValue(pos-2,5,index);
            //--- form the intermediate bit-map
            map=0;
            if(m_extr_pr[i-2]>m_extr_pr[i])
               map+=1;  // set bit 0
            if(m_extr_osc[i-2]>m_extr_osc[i])
               map+=4;  // set bit 2
            //--- add the result
            m_extr_map+=map<<(4*(i-2));
           }
         else
            m_extr_pr[i]=m_high.MaxValue(pos-1,4,index);
        }
     }
//---
   ArraySetAsSeries(_ma1, true);
  // ArraySetAsSeries(_state, true);
   if (CopyBuffer(ci_handle1,0,0,20,_ma1) < 0){Print("CopyBufferMA1 error =",GetLastError());}
  // if (CopyBuffer(ci_macd2_handle,0,0,20,_ma1) < 0){Print("CopyBufferMA2 error =",GetLastError());}
  // if (CopyBuffer(ci_handle1,0,0,20,_state) < 0){Print("CopyBufferState1 error =",GetLastError());}
   //if (CopyBuffer(ci_macd2_handle,0,0,20,_state) < 0)   {
    //  Print("CopyBufferATR error =",GetLastError());}
   return(true);
  }

//+------------------------------------------------------------------+
//| Comparing the bit-map of extremums with pattern.                 |
//+------------------------------------------------------------------+
bool CSignalSAR::CompareMaps(int map,int count,bool minimax,int start)
  {
   int step =(minimax)?4:8;
   int total=step*(start+count);
//--- check input parameters for a possible going out of range of the bit-map
   if(total>32)
      return(false);
//--- bit-map of the patter is an "array" of 4-bit fields
//--- each "element of the array" definitely describes the desired ratio
//--- of current extremums of the oscillator and the price with previous ones
//--- purpose of bits of an elements of the pattern of the bit-map pattern
//--- bit 3 - is equal to if the ratio of extremums of the oscillator is insignificant for us
//---         is equal to 0 if we want to "find" the ratio of extremums of the oscillator determined by the value of bit 2
//--- bit 2 - is equal to 1 if we want to "discover" the situation when the current extremum of the "oscillator" is "more extreme" than the previous one
//---         (current peak is higher or current valley is deeper)
//---         is equal to 0 if we want to "discover" the situation when the current extremum of the oscillator is "less extreme" than the previous one
//---         (current peak is lower or current valley is less deep)
//--- bit 1 - is equal to 1 if the ratio of extremums is insignificant for us
//---         it is equal to 0 if we want to "find" the ratio of price extremums determined by the value of bit 0
//--- bit 0 - is equal to 1 if we want to "discover" the situation when the current price extremum is "more extreme" than the previous one
//---         (current peak is higher or current valley is deeper)
//---         it is equal to 0 if we want to "discover" the situation when the current price extremum is "less extreme" than the previous one
//---         (current peak is lower or current valley is less deep)
   uint inp_map,check_map;
   int  i,j;
//--- loop by extremums (4 minimums and 4 maximums)
//--- price and the oscillator are checked separately (thus, there are 16 checks)
   for(i=step*start,j=0;i<total;i+=step,j+=4)
     {
      //--- "take" two bits - patter of the corresponding extremum of the price
      inp_map=(map>>j)&3;
      //--- if the higher-order bit=1, then any ratio is suitable for us
      if(inp_map<2)
        {
         //--- "take" two bits of the corresponding extremum of the price (higher-order bit is always 0)
         check_map=(m_extr_map>>i)&3;
         if(inp_map!=check_map)
            return(false);
        }
      //--- "take" two bits - pattern of the corresponding oscillator extremum
      inp_map=(map>>(j+2))&3;
      //--- if the higher-order bit=1, then any ratio is suitable for us
      if(inp_map>=2)
         continue;
      //--- "take" two bits of the corresponding oscillator extremum (higher-order bit is always 0)
      check_map=(m_extr_map>>(i+2))&3;
      if(inp_map!=check_map)
         return(false);
     }
//--- ok
   return(true);
  }


//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Generating a signal for reversing a short position               |
//+------------------------------------------------------------------+
ENUM_MT_POLICY CSignalSAR::MTF_CheckReverseLong(double &price,double &sl,double &tp,datetime &expiration)
  {
  
  int result=0;
  int idx   =StartIndex();
  m1_state=P0_CHECK;
    if( IS_PATTERN_USAGE(1) && m_period_op< PERIOD_M30)
     {
        
         if ( flag_sar_count  > 0 && small_change_flag!=flag_sar_count){   //       ( small_sar_up  > small_sar_down && small_sar_down )
           small_change_flag=   flag_sar_count ;
           if(big_sar_up != EMPTY_VALUE){   
               m1_state=P3_Rev_L;
         }}

      /*int check2;
       if( Signal(idx)> 0 && Main(idx)< Signal(idx) && Main(idx+2)>= Signal(idx+2) ) //V1 DiffMain(idx)<0.0
           check2=P3_Rev_L;  
       if(  m1_state!=P3_Rev_L &&  check2==P3_Rev_L )    
               Print("Need add filter") ;
       */            
     }
      //--- main line is directed downwards, confirming a possibility of falling of price
      
     if(IS_PATTERN_USAGE(2))
      {  //if(Signal(idx)< 0 && Main(idx)< Signal(idx)&& Main(idx+1)>= Signal(idx+2) )
         //   m1_state=P4_Con_S;
      //--- if the model 1 is used, look for a reverse of the main line
         
      }
	 
	 if(IS_PATTERN_USAGE(3))   
     {//--- the main line is directed upwards, and it confirms the possibility of price growth
		// if(DiffMain(idx)>0.0 && (Main(idx)<0.0 && Main(idx+1)>0.0) )
       //  m1_state=P6_Cross_S;      // "confirming" signal number 0
      //--- if the model 1 is used, look for a reverse of the main line
     }
      expiration+=m_expiration*PeriodSeconds(m_period);
     //  if(!OpenShortParams(price,sl,tp,expiration))
     //      return(false);	 
    //   
      return (m1_state);    
}
ENUM_MT_POLICY CSignalSAR::MTF_CheckReverseShort(double &price,double &sl,double &tp,datetime &expiration)
  {
  int result=0;
  int idx   =StartIndex();
  m1_state=P0_CHECK;
   if( IS_PATTERN_USAGE(1) && m_period_op< PERIOD_M30)
     {
       
          if ( flag_sar_count  < 0 && small_change_flag!=flag_sar_count){   //       ( small_sar_up  > small_sar_down && small_sar_down )
           small_change_flag=   flag_sar_count ;
           if(big_sar_down != EMPTY_VALUE){  
               m1_state=P1_Rev_S;
          }}  
 

              
     // if( Signal(idx)> 0 && Main(idx)< Signal(idx) && Main(idx+1)>= Signal(idx+1) ) //V1 DiffMain(idx)<0.0
   //   if( Main(idx)> Signal(idx) && Signal(idx)< 0  && Main(idx+1)<= Signal(idx+1)) //V1 DiffMain(idx)<0.0
		 //   m1_state=P1_Rev_S;
	  /*    int check2 ;
	  if  ( Main(idx)> Signal(idx) && Signal(idx)< 0  && Main(idx+2)<= Signal(idx+2)) 
		       check2 = P1_Rev_S; 
		if(m1_state!=P1_Rev_S &&check2 ==P1_Rev_S )   
		   Print("Need add filter") ;
		*/       
     }
      //--- main line is directed downwards, confirming a possibility of falling of price
      
     if(IS_PATTERN_USAGE(2))
      { 
    //   if(Signal(idx)> 0 && Main(idx)> Signal(idx) &&  Main(idx+1)<= Signal(idx+2))
      //   m1_state=P2_Con_L;
      }
	  
	  if(IS_PATTERN_USAGE(3))   //filter size
     {
      //--- the main line is directed upwards, and it confirms the possibility of price growth
	//	 if(DiffMain(idx)>0.0 &&  (Main(idx)>0.0 && Main(idx+1)<0.0) )
      //   m1_state=P5_Cross_L;      // "confirming" signal number 0
      //--- if the model 1 is used, look for a reverse of the main line
     }
      
    //   if(!OpenLongParams(price,sl,tp,expiration))
     //      return(false);	 
    //   else
     expiration+=m_expiration*PeriodSeconds(m_period);
     return (m1_state);    
    }  
  



//+------------------------------------------------------------------+
//| Generating a buy signal                                          |
//+------------------------------------------------------------------+
bool CSignalSAR::CheckOpenLong(double &price,double &sl,double &tp,datetime &expiration)
  {
   bool   result   =false;
//--- the "prohibition" signal
   if(m_direction==EMPTY_VALUE)
      return(false);
//--- check of exceeding the threshold value
   if(m_direction>=m_threshold_open)
     {
      //--- there's a signal
      result=true;
      //--- try to get the levels of opening
      expiration+=m_expiration*PeriodSeconds(m_period);
      
     // if(!OpenLongParams(price,sl,tp,expiration))
         result=false;
     }
//--- zeroize the base price
   m_base_price=0.0;
//--- return the result
   return(result);
  }
//+------------------------------------------------------------------+
//| Generating a sell signal                                         |
//+------------------------------------------------------------------+
bool CSignalSAR::CheckOpenShort(double &price,double &sl,double &tp,datetime &expiration)
  {
   bool   result   =false;
//--- the "prohibition" signal
   if(m_direction==EMPTY_VALUE)
      return(false);
//--- check of exceeding the threshold value
   if(-m_direction>=m_threshold_open)
     {
      //--- there's a signal
      result=true;
      //--- try to get the levels of opening
     // if(!OpenShortParams(price,sl,tp,expiration))
     expiration+=m_expiration*PeriodSeconds(m_period);
      //   result=false;
     }
//--- zeroize the base price
   m_base_price=0.0;
//--- return the result
   return(result);
  }




bool CSignalSAR::CheckCloseLong(double &price)
  {

//--- return the result
   return(true);
  }
//+------------------------------------------------------------------+
//| Generating a signal for closing a short position                 |
//+------------------------------------------------------------------+
bool CSignalSAR::CheckCloseShort(double &price)
  {
   bool   result   =false;
//--- the "prohibition" signal
   if(m_direction==EMPTY_VALUE)
      return(false);
//--- check of exceeding the threshold value
   if(m_direction>=m_threshold_close)
     {
      //--- there's a signal
      result=true;
      //--- try to get the level of closing
      if(!CloseShortParams(price))
         result=false;
     }
//--- zeroize the base price
   m_base_price=0.0;
//--- return the result
   return(result);
  }

bool CSignalSAR::CloseLong(double &price){
      bool   result   =false;
   if(m1_state==MTP_REVSHORT)
   {
   //--- try to get the level of closing
   //  if(Signal(2)>Signal(0) && Signal(0)>0 ){

           m_base_price=0.0;
   	      if(CloseLongParams(price))
               result=true;
//--- zeroize the base price
   }
   return(result); 
}
bool CSignalSAR::CloseShort(double &price){
   bool   result   =false;
   if(m1_state==MTP_REVLONG)
   {
      double sl=0,tp=0;
      datetime expiration=0;
   //   if(Signal(2)<Signal(0)&& Signal(0)<0 ){

	      if(CloseShortParams(price))
           result=true;	  
	//   } }
   }  
   m_base_price=0.0;
   return(result); 
}
